# -*- coding: utf-8 -*-
import datetime

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.conf.urls.defaults import *
from django.forms.util import ErrorList
from django import forms
from django.http import Http404, HttpResponse, HttpResponseRedirect, HttpResponseForbidden
from django.contrib.admin.widgets import AdminDateWidget
from django.db.models import Q
from django.db import connection
from django.conf import settings
from django.utils.formats import number_format

from jumbo.forms.models import AdminModelForm
from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.middleware.thread_local import get_request
from jumbo.apps.smart_selects.widgets import ChainedSelect
from jumbo.apps.autocomplete import AutocompleteModelChoiceField
from jumbo.apps.autocomplete import autocomplete
from jumbo.shortcuts import render_to_response

from expense.models import *
from organization.models import Contact

########################################
# FORM INLINE                          #
########################################
#class PrivateExpensiveInlineForm(AdminModelForm):
#    class Meta:
#        model = PrivateExpensive

    #def clean(self):
    #    cleaned_data = self.cleaned_data
    #    time_begin = cleaned_data.get('time_begin')
    #    time_end = cleaned_data.get('time_end')
    #
    #    ## verifico time_begin e time_end completamente vuoti o pieni
    #    if (time_begin or time_end) and (not time_begin or not time_end):
    #        if not time_begin:
    #            self._errors["time_begin"] = ErrorList(["Complete Time"])
    #            del cleaned_data["time_begin"]
    #        if not time_end:
    #            self._errors["time_end"] = ErrorList(["Complete Time"])
    #            del cleaned_data["time_end"]
    #    return cleaned_data

class TransportationInlineForm(AdminModelForm):
    class Meta:
        model = Transportation

    def clean(self):
        cleaned_data = self.cleaned_data
        time_begin = cleaned_data.get('time_begin')
        time_end = cleaned_data.get('time_end')

        ## verifico time_begin e time_end completamente vuoti o pieni
        if (time_begin or time_end) and (not time_begin or not time_end):
            if not time_begin:
                self._errors["time_begin"] = ErrorList(["Complete Time"])
                del cleaned_data["time_begin"]
            if not time_end:
                self._errors["time_end"] = ErrorList(["Complete Time"])
                del cleaned_data["time_end"]
        return cleaned_data

class AutoTravelInlineForm(AdminModelForm):
    class Meta:
        model = AutoTravel

    def clean(self):
        cleaned_data = self.cleaned_data
        time_begin = cleaned_data.get('time_begin')
        time_end = cleaned_data.get('time_end')

        ## verifico time_begin e time_end completamente vuoti o pieni
        if (time_begin or time_end) and (not time_begin or not time_end):
            if not time_begin:
                self._errors["time_begin"] = ErrorList(["Complete Time"])
                del cleaned_data["time_begin"]
            if not time_end:
                self._errors["time_end"] = ErrorList(["Complete Time"])
                del cleaned_data["time_end"]
        return cleaned_data

class MealInlineForm(AdminModelForm):
    class Meta:
        model = Meal

    def clean(self):
        cleaned_data = self.cleaned_data
        time_begin = cleaned_data.get('time_begin')
        time_end = cleaned_data.get('time_end')

        ## verifico time_begin e time_end completamente vuoti o pieni
        if (time_begin or time_end) and (not time_begin or not time_end):
            if not time_begin:
                self._errors["time_begin"] = ErrorList(["Complete Time"])
                del cleaned_data["time_begin"]
            if not time_end:
                self._errors["time_end"] = ErrorList(["Complete Time"])
                del cleaned_data["time_end"]
        return cleaned_data

class LodgingInlineForm(AdminModelForm):
    class Meta:
        model = Lodging

    def clean(self):
        cleaned_data = self.cleaned_data
        time_begin = cleaned_data.get('time_begin')
        time_end = cleaned_data.get('time_end')

        ## verifico time_begin e time_end completamente vuoti o pieni
        if (time_begin or time_end) and (not time_begin or not time_end):
            if not time_begin:
                self._errors["time_begin"] = ErrorList(["Complete Time"])
                del cleaned_data["time_begin"]
            if not time_end:
                self._errors["time_end"] = ErrorList(["Complete Time"])
                del cleaned_data["time_end"]
        return cleaned_data

class ClientEntertainmentInlineForm(AdminModelForm):
    class Meta:
        model = ClientEntertainment

    def clean(self):
        cleaned_data = self.cleaned_data
        time_begin = cleaned_data.get('time_begin')
        time_end = cleaned_data.get('time_end')
        client_name = cleaned_data.get('client_name')
        client_company = cleaned_data.get("client_company")

        ## verifico time_begin e time_end completamente vuoti o pieni
        if (time_begin or time_end) and (not time_begin or not time_end):
            if not time_begin:
                self._errors["time_begin"] = ErrorList(["Complete Time"])
                del cleaned_data["time_begin"]
            if not time_end:
                self._errors["time_end"] = ErrorList(["Complete Time"])
                del cleaned_data["time_end"]

        ## verifico completezza dati
        if not client_name and not client_company:
            if not client_name:
                self._errors["client_name"] = ErrorList(["Complete Client Name or Client Company"])
                del cleaned_data["client_name"]
            if not client_company:
                self._errors["client_company"] = ErrorList(["Complete Client Name or Client Company"])
                del cleaned_data["client_company"]
        return cleaned_data

########################################
# INLINE                               #
########################################
class TransportationInline(admin.TabularInline):
    model  = Transportation
    form   = TransportationInlineForm
    extra  = 0
    fields = ('date_begin', 'date_end', 'time_begin', 'time_end', 'location', 'air_cost', 'bus_cost', 'train_cost', 'taxi_cost')

class AutoTravelInline(admin.TabularInline):
    model  = AutoTravel
    form   = AutoTravelInlineForm
    extra  = 0
    fields = ('date_begin', 'date_end', 'time_begin', 'time_end', 'location', 'car', 'car_km', 'rate_per_km', 'rental_cost', 'other_cost')

class MealInline(admin.TabularInline):
    model  = Meal
    form   = MealInlineForm
    extra  = 0
    fields = ('date_begin', 'date_end', 'time_begin', 'time_end', 'location', 'breakfast_cost', 'lunch_cost', 'dinner_cost', 'other_cost')

class LodgingInline(admin.TabularInline):
    model  = Lodging
    form   = LodgingInlineForm
    extra  = 0
    fields = ('date_begin', 'date_end', 'time_begin', 'time_end', 'location', 'lodging_cost', 'phone_cost', 'tip_cost', 'laundry_cost', 'meal_cost', 'other_cost')

class ClientEntertainmentInline(admin.TabularInline):
    model  = ClientEntertainment
    form   = ClientEntertainmentInlineForm
    extra  = 0
    fields = ('date_begin', 'date_end', 'time_begin', 'time_end', 'location', 'client_name', 'client_company', 'restaurant_cost', 'sport_cost', 'lodging_cost', 'air_cost', 'other_cost')

class PrivateExpensiveInline(admin.TabularInline):
    model  = PrivateExpensive
    extra  = 0
    fields = ('date_begin', 'location', 'type_of_expensive', 'cost', 'excess_as_per_policy')

class CompanyExpressInline(admin.TabularInline):
    model  = CompanyExpress
    #form   = PrivateExpensiveInlineForm
    extra  = 0
    fields = ('date_begin', 'supplier', 'description', 'invoice',)

class OtherExpenseInline(admin.TabularInline):
    model  = OtherExpense
    #form   = PrivateExpensiveInlineForm
    extra  = 0
    fields = ('date_begin', 'supplier', 'description', 'invoice', 'seminar_conference')

########################################
# SEARCH                               #
########################################
TYPE_FIELDS_CHOICES = (
    ('', ("---------")),
    ('expense_transportation.air_cost', ("Air Travel")),
    ('expense_transportation.bus_cost', _("Bus/Subway")),
    ('expense_transportation.train_cost', _("Train")),
    ('expense_transportation.taxi_cost', _("Taxi")),
    ('expense_autotravel.car_km', _("Car Km")),
    ('expense_autotravel.rate_per_km', _("Rate Per Km")),
    ('expense_autotravel.rental_cost', _("Rental Cost")),
    ('expense_autotravel.other_cost', _("Car Travel Other Cost")),
    ('expense_meal.breakfast_cost', _("Breakfast")),
    ('expense_meal.lunch_cost', _("Lunch")),
    ('expense_meal.dinner_cost', _("Dinner")),
    ('expense_meal.other_cost', _("Meal Other Cost")),
    ('expense_lodging.lodging_cost', _("Apartment/hotel")),
    ('expense_lodging.phone_cost', _("Phone")),
    ('expense_lodging.tip_cost', _("Tips")),
    ('expense_lodging.laundry_cost', _("Laundry")),
    ('expense_lodging.meal_cost', _("Lodging Meal")),
    ('expense_lodging.other_cost', _("Lodging Other Cost")),
    ('expense_cliententertainment.restaurant_cost', _("Restaurants")),
    ('expense_cliententertainment.sport_cost', _("Sports/Exibitions")),
    ('expense_cliententertainment.lodging_cost', _("Lodging")),
    ('expense_cliententertainment.air_cost', _("Client Ent. Airtravel")),
    ('expense_cliententertainment.other_cost', _("Gift/Other")),
    ('expense_privateexpensive.cost', _("Private Cost")),
    ('expense_privateexpensive.excess_as_per_policy', _("Excess as per policy")),
    ('expense_companyexpress.invoice', _("Company Expense - Receipt/Invoice")),
    ('expense_otherexpense.invoice', _("Other Expense - Receipt/Invoice")),
    ('expense_otherexpense.seminar_conference', _("Other Expense - Seminars/Conferences")),
)
TYPE_FIELDS_DATE_RECEIPT_CHOICES = (
    ('', ("---------")),
    ('expense_transportation', ("Transport")),
    ('expense_autotravel', ("Car Expenses")),
    ('expense_meal', ("Meals")),
    ('expense_lodging', ("Lodging and Meals")),
    ('expense_cliententertainment', ("Client Entertainments")),
    ('expense_privateexpensive', ("Private Expenses")),
    ('expense_companyexpress', ("Company Expenses")),
    ('expense_other', ("Other Expenses")),
)
TOTAL_TYPE_FIELDS_CHOICES = (
    ('', ("---------")),
    ('calc_transportation_total', "Transport"),
    ('calc_autotravel_total', "Car Expenses"),
    ('calc_meal_total', "Meals"),
    ('calc_lodging_total', "Lodging and Meals"),
    ('calc_client_entertainment_total', "Client Entertainments"),
    ('calc_private_expensive_total', "Private Expenses"),
    ('calc_company_express_total',"Company Expenses"),
    ('calc_other_expense_total',"Other Expenses"),
    ('calc_total_expense', "Total Expense")
)

WHAT_CURRENCY_CHOICES = (
    ('', ("---------")),
    ('document_currency', ("Document Currency")),
    ('destination_currency', _("Destination Currency")),
)

class ExpenseSearchForm(forms.ModelForm):
    class Meta:
        model          = Expense
        display_fields = (
            ('assigned_to__last_name__istartswith', 'assigned_to__first_name__istartswith'),
            ('date_begin__gte', 'date_end__lte'),
            ('type_fields_date_receipt', 'date_receipt__gte', 'date_receipt__lte'),
            ('number__gte', 'number__lte'),
            'code',
            ('currency',),
            ('type_fields', 'what_currency', 'amount__gte', 'amount__lte'),
            ('totals_type_fields', 'totals_what_currency', 'totals_amount__gte', 'totals_amount__lte'),
            ('authorized_by_manager', 'authorized_by_administration')
        )

    def __init__(self, *args, **kws):
        super(ExpenseSearchForm, self).__init__(*args, **kws)
        BOOLEAN = (('', '---'), ('1', _('Yes')), ('0', _('No')),)

        self.fields['assigned_to__last_name__istartswith'] = forms.CharField(
            label=_("Assigned To Last Name"),
            required=False,
            help_text=_('Last name starts with')
        )
        self.fields['assigned_to__first_name__istartswith'] = forms.CharField(
            label=_("Assigned To First Name"),
            required=False,
            help_text=_('First name starts with')
        )
        self.fields['date_begin__gte'] = forms.DateField(label =_("Date Begin >="), required =False,
                                                         widget = AdminDateWidget)
        self.fields['date_end__lte'] = forms.DateField(label=_("Date End <="), required=False,
                                                       widget = AdminDateWidget)
        self.fields['type_fields_date_receipt'] = forms.ChoiceField(label = 'Date Receipt',
                                                                    choices=TYPE_FIELDS_DATE_RECEIPT_CHOICES,
                                                                    required=False)
        self.fields['date_receipt__gte'] = forms.DateField(label =_("Date Receipt >="), required =False,
                                                         widget = AdminDateWidget)
        self.fields['date_receipt__lte'] = forms.DateField(label=_("Date Receipt <="), required=False,
                                                       widget = AdminDateWidget)
        self.fields['number__gte'] = forms.IntegerField(label=_("From Number"), required=False)
        self.fields['number__lte'] = forms.IntegerField(label=_("To Number"), required=False)
        self.fields['type_fields'] = forms.ChoiceField(choices=TYPE_FIELDS_CHOICES, required=False)
        self.fields['what_currency'] = forms.ChoiceField(choices=WHAT_CURRENCY_CHOICES, required=False)
        self.fields['amount__gte'] = forms.DecimalField(label=_("Amount >="), required=False)
        self.fields['amount__lte'] = forms.DecimalField(label=_("Amount <="), required=False)

        # TOTALI
        self.fields['totals_type_fields'] = forms.ChoiceField(label=_("Type Fields"), choices=TOTAL_TYPE_FIELDS_CHOICES, required=False)
        self.fields['totals_what_currency'] = forms.ChoiceField(choices=WHAT_CURRENCY_CHOICES, required=False)
        self.fields['totals_amount__gte'] = forms.DecimalField(label=_("Total Amount >="), required=False)
        self.fields['totals_amount__lte'] = forms.DecimalField(label=_("Total Amount <="), required=False)

        self.fields['authorized_by_manager'] = forms.ChoiceField(
            label=_("Authorized by manager"),
            choices=BOOLEAN,
            required=False
        )
        self.fields['authorized_by_administration'] = forms.ChoiceField(
            label=_("Authorized by administration"),
            choices=BOOLEAN,
            required=False
        )

    def clean(self):
        cleaned_data = self.cleaned_data
        amount_gte = cleaned_data.get("amount__gte")
        amount_lte = cleaned_data.get("amount__lte")
        type_fields = cleaned_data.get("type_fields")

        if amount_gte:
            if not type_fields:
                msg = "Field 'type_fields' is required if 'amount_gte' is not empty"
                self._errors["amount__gte"] = ErrorList([msg])
                del cleaned_data["amount_gte"]

        if amount_lte:
            if not type_fields:
                msg = "Field 'type_fields' is required if 'amount_lte' is not empty"
                self._errors["amount__lte"] = ErrorList([msg])
                del cleaned_data["amount_lte"]

        return cleaned_data

    def change_search_filter(self, lookup_params, qs):
        #########################################
        # Ricerca per data scontrino
        #########################################
        type_fields_date_receipt = None
        if lookup_params.has_key('type_fields_date_receipt'):
            type_fields_date_receipt = lookup_params['type_fields_date_receipt']
            del lookup_params['type_fields_date_receipt']

        if lookup_params.has_key('date_receipt__gte'):
            date_receipt_gte = lookup_params['date_receipt__gte']
            q_gte = Q()
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_transportation':
                q_gte = q_gte | Q(transportations__date_begin__gte=date_receipt_gte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_autotravel':
                q_gte = q_gte | Q(autotravels__date_begin__gte=date_receipt_gte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_meal':
                q_gte = q_gte | Q(meals__date_begin__gte=date_receipt_gte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_lodging':
                q_gte = q_gte | Q(lodgings__date_begin__gte=date_receipt_gte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_cliententertainment':
                q_gte = q_gte | Q(client_entertainments__date_begin__gte=date_receipt_gte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_privateexpensive':
                q_gte = q_gte | Q(privates__date_begin__gte=date_receipt_gte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_companyexpress':
                q_gte = q_gte | Q(company_expresses__date_begin__gte=date_receipt_gte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_other':
                q_gte = q_gte | Q(other_expenses__date_begin__gte=date_receipt_gte)

            if not lookup_params.has_key('date_receipt__lte'):
                del lookup_params['date_receipt__gte']
                qs = qs.filter(q_gte).distinct()

        if lookup_params.has_key('date_receipt__lte'):
            date_receipt_lte = lookup_params['date_receipt__lte']
            q_lte = Q()
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_transportation':
                q_lte = q_lte | Q(transportations__date_begin__lte=date_receipt_lte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_autotravel':
                q_lte = q_lte | Q(autotravels__date_begin__lte=date_receipt_lte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_meal':
                q_lte = q_lte | Q(meals__date_begin__lte=date_receipt_lte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_lodging':
                q_lte = q_lte | Q(lodgings__date_begin__lte=date_receipt_lte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_cliententertainment':
                q_lte = q_lte | Q(client_entertainments__date_begin__lte=date_receipt_lte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_privateexpensive':
                q_lte = q_lte | Q(privates__date_begin__lte=date_receipt_lte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_companyexpress':
                q_lte = q_lte | Q(company_expresses__date_begin__lte=date_receipt_lte)
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_other':
                q_lte = q_lte | Q(other_expenses__date_begin__lte=date_receipt_lte)

            if not lookup_params.has_key('date_receipt__gte'):
                del lookup_params['date_receipt__lte']
                qs = qs.filter(q_lte).distinct()

        if lookup_params.has_key('date_receipt__gte') and lookup_params.has_key('date_receipt__lte'):
            del lookup_params['date_receipt__gte']
            del lookup_params['date_receipt__lte']
            qq = Q()
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_transportation':
                qq = qq | (Q(transportations__date_begin__gte=date_receipt_gte) & Q(transportations__date_begin__lte=date_receipt_lte))
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_autotravel':
                qq = qq | (Q(autotravels__date_begin__gte=date_receipt_gte) & Q(autotravels__date_begin__lte=date_receipt_lte))
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_meal':
                qq = qq | (Q(meals__date_begin__gte=date_receipt_gte) & Q(meals__date_begin__lte=date_receipt_lte))
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_lodging':
                qq = qq | (Q(lodgings__date_begin__gte=date_receipt_gte) & Q(lodgings__date_begin__lte=date_receipt_lte))
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_cliententertainment':
                qq = qq | (Q(client_entertainments__date_begin__gte=date_receipt_gte) & Q(client_entertainments__date_begin__lte=date_receipt_lte))
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_privateexpensive':
                qq = qq | (Q(privates__date_begin__gte=date_receipt_gte) & Q(privates__date_begin__lte=date_receipt_lte))
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_companyexpress':
                qq = qq | (Q(company_expresses__date_begin__gte=date_receipt_gte) & Q(company_expresses__date_begin__lte=date_receipt_lte))
            if not type_fields_date_receipt or type_fields_date_receipt == 'expense_other':
                qq = qq | (Q(other_expenses__date_begin__gte=date_receipt_gte) & Q(other_expenses__date_begin__lte=date_receipt_lte))
            qs = qs.filter(qq)
        qs = qs.distinct()
        #########################################
        # Fine ricerca per data scontrino
        #########################################

        #########################################
        # Ricerca per totali Expense
        #########################################
        totals_type_fields = None
        if lookup_params.has_key('totals_type_fields'):
            totals_type_fields = lookup_params['totals_type_fields']
            del lookup_params['totals_type_fields']

        totals_what_currency= None
        if lookup_params.has_key('totals_what_currency'):
            totals_what_currency = lookup_params['totals_what_currency']
            del lookup_params['totals_what_currency']

        if lookup_params.has_key('totals_amount__gte'):
            totals_amount__gte = lookup_params['totals_amount__gte']
            totals_extra_where = "(1=1"
            if totals_type_fields in ('calc_transportation_total', 'calc_autotravel_total', 'calc_meal_total',
                                      'calc_lodging_total', 'calc_client_entertainment_total', 'calc_private_expensive_total',
                                      'calc_company_express_total', 'calc_other_expense_total', 'calc_total_expense'):
                if totals_what_currency == 'destination_currency':
                    totals_extra_where += ' and %s * expense_expense.change >= %s)' % (totals_type_fields, totals_amount__gte)
                else:
                    totals_extra_where += ' and %s >= %s)' % (totals_type_fields, totals_amount__gte)

            del lookup_params['totals_amount__gte']
            qs = qs.extra(where  = [totals_extra_where]).distinct()

        if lookup_params.has_key('totals_amount__lte'):
            totals_amount__lte = lookup_params['totals_amount__lte']

            totals_extra_where = "(1=1"
            if totals_type_fields in ('calc_transportation_total', 'calc_autotravel_total', 'calc_meal_total',
                                      'calc_lodging_total', 'calc_client_entertainment_total', 'calc_private_expensive_total',
                                      'calc_company_express_total', 'calc_other_expense_total', 'calc_total_expense'):
                if totals_what_currency == 'destination_currency':
                    totals_extra_where += ' and %s * expense_expense.change <= %s)' % (totals_type_fields, totals_amount__lte)
                else:
                    totals_extra_where += ' and %s <= %s)' % (totals_type_fields, totals_amount__lte)

            del lookup_params['totals_amount__lte']
            qs = qs.extra(where  = [totals_extra_where]).distinct()
        #########################################
        # FINE Ricerca per totali Expense
        #########################################

        if not lookup_params.has_key('amount__gte') and not lookup_params.has_key('amount__lte'):
            return (lookup_params, qs)

        type_fields = None
        if lookup_params.has_key('type_fields'):
            type_fields = lookup_params['type_fields']
            del lookup_params['type_fields']

        what_currency= None
        if lookup_params.has_key('what_currency'):
            what_currency = lookup_params['what_currency']
            del lookup_params['what_currency']

        extra_where = ""
        exra_fields = []
        extra_table = []

        # MI SERVE PER FARE POI LE LEFT OUTER JOIN PIU SOTTO
        qs.query.join((None, 'expense_expense', None, None))

        if type_fields:
            extra_table = [type_fields.split(".")[0]]
            exra_fields = [type_fields]
        else:
            extra_table = set()
            for tf in TYPE_FIELDS_CHOICES[1:]:
                extra_table.add(tf[0].split(".")[0])
                exra_fields.append(tf[0])

        for et in extra_table:
            # CON QUESTO COMANDO CREO DELLE LEFT OUTER JOIN SULLE VARIE TABELLE
            # SU CUI DOVRO FARE I VARI OR
            qs.query.join(connection=('expense_expense', et, 'id', 'expense_id'),
                          always_create=True, promote=True)

        extra_where += "((1 = 1"
        for i, tf in enumerate(exra_fields):
            if i > 0:
                extra_where += " or (1 = 1"

            if lookup_params.has_key('amount__gte'):
                amount = lookup_params['amount__gte']
                if what_currency == 'destination_currency':
                    extra_where += ' and %s * expense_expense.change >= %s' % (tf, amount)
                else:
                    extra_where += ' and %s >= %s' % (tf, amount)

            if lookup_params.has_key('amount__lte'):
                amount = lookup_params['amount__lte']
                if what_currency == 'destination_currency':
                    extra_where += ' and %s * expense_expense.change <= %s' % (tf, amount)
                else:
                    extra_where += ' and %s <= %s' % (tf, amount)
            extra_where += ")"
        print extra_where
        extra_where += ")"

        if lookup_params.has_key('amount__gte'):
            del lookup_params['amount__gte']

        if lookup_params.has_key('amount__lte'):
            del lookup_params['amount__lte']

        qs = qs.extra(where  = [extra_where]).distinct()

        return (lookup_params, qs)


########################################
# FORM                                 #
########################################
class ExpenseAdminForm(AdminModelForm):
    class Meta:
        model = Expense

    def __init__(self, *args, **kws):
        super(ExpenseAdminForm, self).__init__(*args, **kws)

    def clean(self):
        cleaned_data = self.cleaned_data
        currency = cleaned_data.get("currency")
        change = cleaned_data.get("change")
        authorized_by_administration = cleaned_data.get("authorized_by_administration")
        if currency.code != "EUR" and not change and authorized_by_administration:
            msg = "Field 'change' is required if 'currency' is not 'Euro' and 'Expense' is authorized"
            self._errors["change"] = ErrorList([msg])
            del cleaned_data["change"]
        return cleaned_data

########################################
# GENERAL                              #
########################################
class ExpenseAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("expense/css/admin_expense.css",)}

    form               = ExpenseAdminForm
    search_form        = ExpenseSearchForm
    inlines            = (TransportationInline, AutoTravelInline, MealInline, LodgingInline, ClientEntertainmentInline, PrivateExpensiveInline,
                          CompanyExpressInline, OtherExpenseInline)
    list_per_page      = 30
    date_hierarchy     = 'date_begin'
    fieldsets          = (
                            (_('logs admin'), {
                                'fields'  : ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
                                'classes': ('collapse',)
                            }),
                            (None, {
                                'fields'  : (
                                    'status',
                                    'number',
                                    ('code', 'assigned_to',),
                                    ('date_begin', 'date_end',),
                                    ('currency', 'destination_currency', 'change',),
                                    ('authorized_by_manager', 'authorized_by_administration',),
                                    'paid_on',
                                    'purpose_trip'
                                )
                            })
                         )

    list_display       = ('get_status_icon', 'number', 'get_detail', 'get_date_begin', 'get_date_end',
                          'currency', 'change', 'authorized_by_manager', 'authorized_by_administration',
                          'get_transportation_total', 'get_autotravel_total', 'get_meal_total',
                          'get_lodging_total', 'get_client_entertainment_total',
                          'get_private_expensive_total', 'get_company_express_total',
                          'get_other_expense_total',
                          'get_totals', 'get_edit_icon', 'get_print_icon', 'get_delete_icon')

    list_display_csv   = ('get_status_display','number', 'assigned_to', 'get_date_begin',
                          'get_date_end', 'currency', 'change', 'get_auth_man',
                          'get_auth_amm', 'euro_transportation_total', 'euro_transportation_total',
                          'euro_meal_total', 'euro_lodging_total',
                          'euro_client_entertainment_total', 'euro_totals')

    default_group_by = 'assigned_to'
    readonly_fields = ('number',)


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(ExpenseAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^export/$'   , self.admin_site.admin_view(self.export_expense)
                               , name='expense_expense_export'),
        )
        return my_urls + urls

    def export_expense(self, request):
        from expense.forms import ExportExpenseForm
        from jumbo.utils.excel import ExcelReport

        if not request.user.has_perm("expense.change_expense"):
            return HttpResponseForbidden("Expense Not Found")

        if request.method == 'POST':
            form = ExportExpenseForm(request.POST)
        else:
            form = ExportExpenseForm()

        if form.is_valid():
            expenses = Expense.objects.filter(date_begin__gte=form.cleaned_data['date_from'],
                                              date_begin__lte=form.cleaned_data['date_to']).order_by('assigned_to__last_name', 'currency')
            if form.cleaned_data['last_name']:
                expenses = expenses.filter(assigned_to__last_name__icontains=form.cleaned_data['last_name'])

            dict_exp = {}
            list_e = []
            list_expenses = []
            for e in expenses:
                #list.add((e.assigned_to, t[0], e.currency))
                totals = e.transportation_total
                dict_exp, list_e = self.get_dict_tot(e, dict_exp, list_e, totals)

                totals = e.autotravel_total
                dict_exp, list_e = self.get_dict_tot(e, dict_exp, list_e, totals)

                totals = e.meal_total
                dict_exp, list_e = self.get_dict_tot(e, dict_exp, list_e, totals)

                totals = e.lodging_total
                dict_exp, list_e = self.get_dict_tot(e, dict_exp, list_e, totals)

                totals = e.client_entertainment_total
                dict_exp, list_e = self.get_dict_tot(e, dict_exp, list_e, totals)

                pet = e.private_expensive_total
                if pet != 0:
                    if not dict_exp.has_key((e.assigned_to, 'private_expensive_total', e.currency)):
                        dict_exp[e.assigned_to, 'private_expensive_total', e.currency] = pet
                        list_e.append([(e.assigned_to, 'private_expensive_total', e.currency),e])
                    else:
                        new_value = dict_exp[e.assigned_to, 'private_expensive_total', e.currency] + pet
                        dict_exp[e.assigned_to, 'private_expensive_total', e.currency] = new_value


                cet = e.company_express_total
                if cet != 0:
                    if not dict_exp.has_key((e.assigned_to, 'company_express_total', e.currency)):
                        dict_exp[e.assigned_to, 'company_express_total', e.currency] = cet
                        list_e.append([(e.assigned_to, 'company_express_total', e.currency),e])
                    else:
                        new_value = dict_exp[e.assigned_to, 'company_express_total', e.currency] + cet
                        dict_exp[e.assigned_to, 'company_express_total', e.currency] = new_value

            excel = ExcelReport()
            excel.add_row(["Name", "Date From", "Date To", "Type Of Expense", "Doc. Currency", "Sum", "Dest. Currency", "Sum"])

            for row in list_e:
                excel.add_row([row[0][0].full_name, form.cleaned_data['date_from'].strftime("%d/%m/%Y"),
                    form.cleaned_data['date_to'].strftime("%d/%m/%Y"), row[0][1].replace('_',' '), row[0][2].name, dict_exp.get(row[0]),
                    row[1].destination_currency.name if row[1].destination_currency else '',
                    dict_exp.get(row[0])*row[1].change if row[1].destination_currency else ''])
            response = HttpResponse(excel.write_report(), mimetype='application/ms-excel')
            response['Content-Disposition'] = 'attachment; filename=export_expense.xls'
            return response


        return render_to_response('admin/expense/expense/export.html', app_label='expense', title="Export Expense", form=form)

    def get_dict_tot(self, e, dict_exp, list_e, totals):
        for t in totals.iteritems():
            if t[0] != 'total':
                if t[1] != 0:
                    if not dict_exp.has_key((e.assigned_to, t[0], e.currency)):
                        dict_exp[e.assigned_to, t[0], e.currency] = t[1]
                        list_e.append([(e.assigned_to, t[0], e.currency), e])
                    else:
                        new_value = dict_exp[e.assigned_to, t[0], e.currency] + t[1]
                        dict_exp[e.assigned_to, t[0], e.currency] = new_value

        return dict_exp, list_e

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "assigned_to":
            user = request.user
            ## Modifico la tendina "assigned to" in funzione della "AZIENDA" associata (category="com")
            ## Modifico la tendina "assigned_to" in funzione dei permessi associati all'utente
            if user.has_perm("expense.add_expense") or user.has_perm("expense.change_expense"):
                kwargs["queryset"] = Contact.objects.filter(status=1, organization__organization_type__category="com")
            elif user.has_perm("expense.add_owner_expense") or user.has_perm("expense.change_owner_expense"):
                kwargs["queryset"] = Contact.objects.filter(user=request.user, organization__organization_type__category="com")
            else:
                kwargs["queryset"] = Contact.objects.none()
            return db_field.formfield(**kwargs)

        return super(ExpenseAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    # FIXME
    ### Modifico il form di ADD/EDIT per eliminare i campi modificabili solo da chi ha permessi particolari
    #def get_form(self, request, obj=None, **kwargs):
        #user = request.user
        ##if not user.has_perm("authorized_as_manager"):
            ##kwargs['exclude'] = list(kwargs.get('exclude', [])) + ['authorized_by_manager']
        ##if not user.has_perm("authorized_as_administration"):
            ##kwargs['exclude'] = list(kwargs.get('exclude', [])) + ['authorized_by_administration']
        #kwargs['exclude'] = list(kwargs.get('exclude', [])) + ['authorized_by_manager']
        #return super(ExpenseAdmin, self).get_form(request, obj, **kwargs)

    # AZIONI #
    def set_authorized_by_manager(self, request, queryset):
        user = request.user
        if not user.has_perm("expense.authorized_as_manager"):
            self.message_user(request, "ATTENTION!!! You can't authorize expenses")
        else:
            rows_updated = queryset.update(authorized_by_manager=True)
            if rows_updated == 1:
                message_bit = "1 object was"
            else:
                message_bit = "%s objects were" % rows_updated
            self.message_user(request, "%s successfully accepted." % message_bit)
    set_authorized_by_manager.short_description = "Set Objects as authorized by manager"


    def reset_authorized_by_manager(self, request, queryset):
        user = request.user
        if not user.has_perm("expense.authorized_as_manager"):
            self.message_user(request, "ATTENTION!!! You can't authorize expenses")
        else:
            rows_updated = queryset.update(authorized_by_manager=False)
            if rows_updated == 1:
                message_bit = "1 object was"
            else:
                message_bit = "%s objects were" % rows_updated
            self.message_user(request, "%s successfully accepted." % message_bit)
    reset_authorized_by_manager.short_description = "Reset Objects as authorized by manager"


    def send_email_authorized_by_administration(self, queryset):
        TO_SEND_MAIL_ON_AUTHORIZED_BY_ADMINISTRATION = get_settings('expense.settings', 'TO_SEND_MAIL_ON_AUTHORIZED_BY_ADMINISTRATION')

        if TO_SEND_MAIL_ON_AUTHORIZED_BY_ADMINISTRATION:
            from_email = get_settings('expense.settings', 'FROM_SEND_MAIL_ON_AUTHORIZED_BY_ADMINISTRATION')
            subject = get_settings('expense.settings', 'SUBJECT_SEND_MAIL_ON_AUTHORIZED_BY_ADMINISTRATION')
            to = TO_SEND_MAIL_ON_AUTHORIZED_BY_ADMINISTRATION

            text_content = render_to_string("admin/expense/expense/mail_on_authorized_by_administration.txt", lista_expense=queryset)
            html_content = render_to_string("admin/expense/expense/mail_on_authorized_by_administration.html", lista_expense=queryset)

            msg = EmailMultiAlternatives(subject, text_content, from_email, to)
            msg.attach_alternative(html_content, "text/html")
            msg.send()


    def set_authorized_by_administration(self, request, queryset):
        ## solo gli oggetti con il 'change' completo vengono autorizzati da ll'amministrazione
        totals = queryset.count()
        queryset = queryset.filter(change__isnull=False)
        user = request.user
        if not user.has_perm("expense.authorized_as_administration"):
            self.message_user(request, "ATTENTION!!! You can't authorize expenses")
        else:
            rows_updated = queryset.update(authorized_by_administration=True)
            if rows_updated == 1:
                message_bit = "1 object of %s was" % totals
            else:
                message_bit = "%s objects of %s were" % (rows_updated, totals)
            self.send_email_authorized_by_administration(queryset)
            self.message_user(request, "%s successfully accepted." % message_bit)
    set_authorized_by_administration.short_description = "Set Objects as authorized by administration"


    def reset_authorized_by_administration(self, request, queryset):
        user = request.user
        if not user.has_perm("expense.authorized_as_administration"):
            self.message_user(request, "ATTENTION!!! You can't authorize expenses")
        else:
            rows_updated = queryset.update(authorized_by_administration=False)
            if rows_updated == 1:
                message_bit = "1 object was"
            else:
                message_bit = "%s objects were" % rows_updated
            self.message_user(request, "%s successfully accepted." % message_bit)
    reset_authorized_by_administration.short_description = "Reset Objects as authorized by administration"

    actions = [set_authorized_by_manager, reset_authorized_by_manager, set_authorized_by_administration, reset_authorized_by_administration]


    def get_auth_man(self, obj):
        return obj.authorized_by_manager
    get_auth_man.boolean = False


    def get_auth_amm(self, obj):
        return obj.authorized_by_administration
    get_auth_amm.boolean = False


    def get_detail(self, obj):
        detail = """<a href="%(href)s" title="%(title)s">%(detail)s</a>""" % {
            'href'  : reverse('admin:expense_expense_detail', obj.id),
            'title' : ugt("View Expense Detail"),
            'detail' : obj.assigned_to
        }
        return detail
    get_detail.short_description = _("Assigned to")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'assigned_to'


    def get_date_begin(self, obj):
        return obj.date_begin.strftime("%d/%m/%Y")
    get_date_begin.short_description = _('Date Begin')
    get_date_begin.admin_order_field = 'date_begin'


    def get_date_end(self, obj):
        return obj.date_end.strftime("%d/%m/%Y")
    get_date_end.short_description = _('Date End')
    get_date_end.admin_order_field = 'date_end'


    def has_add_permission(self, request, obj=None):
        user = request.user
        if user.has_perm("expense.add_expense") or user.has_perm("expense.add_owner_expense"):
            return True
        return False


    def has_change_permission(self, request, obj=None):
        user = request.user
        if obj and obj.can_be_modified:
            return True
        return super(ExpenseAdmin, self).has_change_permission(request, obj)


    def has_delete_permission(self, request, obj=None):
        user = request.user
        if obj and obj.can_be_deleted:
            return True
        return super(ExpenseAdmin, self).has_delete_permission(request, obj)


    def get_transportation_total(self, obj):
        if obj.change and obj.destination_currency:
            return "%s %s" % (obj.destination_currency.symbol, number_format(obj.euro_transportation_total, 2))
        return "%s %s" % (obj.currency.symbol, number_format(obj.transportation_total['total'], 2))
    get_transportation_total.allow_tags = True
    get_transportation_total.short_description = _('Transport')


    def get_autotravel_total(self, obj):
        if obj.change and obj.destination_currency:
            return "%s %s" % (obj.destination_currency.symbol, number_format(obj.euro_autotravel_total, 2))
        return "%s %s" % (obj.currency.symbol, number_format(obj.autotravel_total['total'], 2))
    get_autotravel_total.allow_tags = True
    get_autotravel_total.short_description = _('Car Travel')


    def get_meal_total(self, obj):
        if obj.change and obj.destination_currency:
            return "%s %s" % (obj.destination_currency.symbol, number_format(obj.euro_meal_total, 2))
        return "%s %s" % (obj.currency.symbol, number_format(obj.meal_total['total'], 2))
    get_meal_total.allow_tags = True
    get_meal_total.short_description = _('Meal')


    def get_lodging_total(self, obj):
        if obj.change and obj.destination_currency:
            return "%s %s" % (obj.destination_currency.symbol, number_format(obj.euro_lodging_total, 2))
        return "%s %s" % (obj.currency.symbol, number_format(obj.lodging_total['total'], 2))
    get_lodging_total.allow_tags = True
    get_lodging_total.short_description = _('Lodging')


    def get_client_entertainment_total(self, obj):
        if obj.change and obj.destination_currency:
            return "%s %s" % (obj.destination_currency.symbol, number_format(obj.euro_client_entertainment_total, 2))
        return "%s %s" % (obj.currency.symbol, number_format(obj.client_entertainment_total['total'], 2))
    get_client_entertainment_total.allow_tags = True
    get_client_entertainment_total.short_description = _('Client Entertainment')


    def get_private_expensive_total(self, obj):
        if obj.change and obj.destination_currency:
            return "%s %s" % (obj.destination_currency.symbol, number_format(obj.euro_private_expensive_total, 2))
        return "%s %s" % (obj.currency.symbol, number_format(obj.private_expensive_total, 2))
    get_private_expensive_total.allow_tags = True
    get_private_expensive_total.short_description = _('Private')


    def get_company_express_total(self, obj):
        if obj.change and obj.destination_currency:
            return "%s %s" % (obj.destination_currency.symbol, number_format(obj.euro_company_express_total, 2))
        return "%s %s" % (obj.currency.symbol, number_format(obj.company_express_total, 2))
    get_company_express_total.allow_tags = True
    get_company_express_total.short_description = _('Company')


    def get_other_expense_total(self, obj):
        if obj.change and obj.destination_currency:
            return "%s %s" % (obj.destination_currency.symbol, number_format(obj.euro_other_expense_total, 2))
        return "%s %s" % (obj.currency.symbol, number_format(obj.other_expense_total['total'], 2))
    get_other_expense_total.allow_tags = True
    get_other_expense_total.short_description = _('Other')


    def get_totals(self, obj):
        if obj.change and obj.destination_currency:
            return "%s %s" % (obj.destination_currency.symbol, number_format(obj.euro_totals, 2))
        return "%s %s" % (obj.currency.symbol, number_format(obj.totals, 2))
    get_totals.allow_tags = True
    get_totals.short_description = _('Totals')

    def get_print_icon(self, result):
        opts = self.model._meta
        return "<a href='%(reverse_url)s?print_expense=1&_popup' target='_blank'><img src='%(url)sadmin/images/fax.png' alt='%(window_title)s' title='%(window_title)s'/></a>" % {
            'reverse_url':reverse('admin:expense_expense_detail', result.id),
            'url':settings.MEDIA_URL,
            'window_title':ugt("Detail Expense")
        }
        return ""
    get_print_icon.allow_tags = True
    get_print_icon.short_description = _("P")
admin.site.register(Expense, ExpenseAdmin)
